#!/bin/sh

# Copyright (C) 2004-2011 QVidium Technologies, Inc.

we_are_done() {
    echo 0 > $DONE

    cd /tmp
    rm -f $IMAGE $MD5SUM install.sh

    sleep 2

    rm -f update.status
    rm -f update.done
    sync
}

# DONE file contains error code when install completes
# 0=ok, need to reboot, 1=ok to try again, 2=serious filesystem error
DONE=/tmp/update.done

cd /tmp

rm -f qvidium.pkg
if [ -f mtd_char_op ] ; then
    rm -f /sbin/mtd_char_op
    cp mtd_char_op /sbin
fi

BR='<br>'

IMAGE=uImage_mips_gz
MD5SUM=$IMAGE.md5

uname -r | grep mipsisa32r2el > /dev/null 2>&1
if [ $? != 0 ] ; then
    echo "This update is not for this system"
    we_are_done
    exit 0
fi

output=`md5sum -c $MD5SUM`
if [ $? != 0 ] ; then
    echo "checksum failed on ${IMAGE}${BR}"
    echo 1 > $DONE
    exit 1
fi
echo "${BR}${output}${BR}"

echo "<br>Stopping watchdog..."
if [ -f /qvidium/rc.watchdog ] ; then
    /qvidium/rc.watchdog stop
elif pidof wd.sh > /dev/null ; then
    REG_VAL=03b0 # for QVPro board
    [ -f /flash/qvidium/inputs.conf ] && . /flash/qvidium/inputs.conf
    [ "$HAS_SDI" = false ] && REG_VAL=03ae # for QVSD board
    i2c -c1 e4 $REG_VAL > /dev/null 2>&1
    killall wd.sh
fi

copy_kernel() {
    NUM=$1
    MTD_DEVICE=$2
    MINOR_NUM=$3
    [ ! -c /dev/$MTD_DEVICE ] && mknod /dev/$MTD_DEVICE c 90 $MINOR_NUM

    echo "${BR}${BR}Erasing kernel $NUM..."
    echo "<pre>"
    echo mtd_char_op erase $MTD_DEVICE
    mtd_char_op erase $MTD_DEVICE
    echo "</pre>"

    echo "${BR}Installing kernel $NUM..."
    echo "<pre>"
    echo mtd_char_op write $MTD_DEVICE $IMAGE
    mtd_char_op write $MTD_DEVICE $IMAGE
    rc=$?
    echo "</pre>"

    rm /dev/$MTD_DEVICE

    if [ $rc != 0 ] ; then
        echo "${BR}Could not update. Do not power cycle or reboot.${BR}"
        echo 2 > $DONE
        exit 1
    fi
}

copy_kernel 0 mtd4 8
copy_kernel 1 mtd5 10

echo "${BR}Update finished."

we_are_done

exit 0
